// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <errno.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <sys/socket.h>

#include <client.h>


client::client() {
}

client::client(char *unixport, int retrytime, int tries) {
	openUnixConnection(unixport,retrytime,tries);
}

client::client(char *hostname, unsigned short int port,
		int retrytime, int tries) {
	openInetConnection(hostname,port,retrytime,tries);
}

client::~client() {
	closeConnection();
}

int	client::openInetConnection(char *hostname, unsigned short int port, 
				int retrytime, int tries) {

	if (!(protocol=getprotobyname("tcp"))) {
		cerr << "error: getprotobyname failed: ";
		cerr << strerror(errno) << endl;
		return 0;
	}

	hostentry=NULL;
	hostentry=gethostbyname(hostname);
	if (!hostentry) {
		cerr << "error: gethostbyname failed: ";
		cerr << strerror(errno) << endl;
		return 0;
	}

	bzero((void *)&serversin,sizeof(serversin));
	serversin.sin_family=hostentry->h_addrtype;
	serversin.sin_port=htons(port);


	// try to connect to each of the addresses that came back
	int	counter=0;
	while (counter<tries) {

		int	addressindex=0;
		while (hostentry->h_addr_list[addressindex]) {

			bcopy((void *)hostentry->h_addr_list[addressindex],
				(void *)&serversin.sin_addr,
				hostentry->h_length);
			addressindex++;
	
			if ((serversock=socket(AF_INET,SOCK_STREAM,
						protocol->p_proto))>-1) {

				if (connect(serversock,
						(struct sockaddr *)&serversin,
						sizeof(serversin))==0) {
					return 1;
				} else {
					cerr << "error: inet connect failed: ";
					cerr << strerror(errno) << endl;
					cerr << "	trying: ";
					cerr << hostname << ":";
					cerr << port << endl;
					closeConnection();
				}
			} else {
				cerr << "error: inet socket creation failed: ";
				cerr << strerror(errno) << endl;
				cerr << "	trying: ";
				cerr << hostname << ":";
				cerr << port << endl;
			}

		}
		counter++;
		sleep(retrytime);
	}
	return 0;
}

int	client::openUnixConnection(char *unixport, int retrytime, int tries) {

	serversun.sun_family=AF_UNIX;
	strcpy(serversun.sun_path,unixport);

	int	counter=0;
	while (counter<tries) {

		if ((serversock=socket(AF_UNIX,SOCK_STREAM,0))>-1) {

			if (connect(serversock,
					(struct sockaddr *)&serversun,
					sizeof(serversun))==0) {
				return 1;
			} else {
				cerr << "error: unix connect failed: ";
				cerr << strerror(errno) << endl;
				cerr << "	trying: " << unixport << endl;
				closeConnection();
			}
		} else {
			cerr << "error: unix socket creation failed: ";
			cerr << strerror(errno) << endl;
			cerr << "	trying: " << unixport << endl;
		}

		counter++;
		sleep(retrytime);
	}
	return 0;
}

void	client::closeConnection() {
	if (serversock>-1) {
		close(serversock);
		serversock=-1;
	}
}

int	client::writeToServer(short number) {
	int	temp=htons(number);
	return (int)write(serversock,(void *)&temp,sizeof(short));
}

int	client::writeToServer(long number) {
	int	temp=htonl(number);
	return (int)write(serversock,(void *)&temp,sizeof(long));
}

int	client::writeToServer(float number) {
	return (int)write(serversock,(void *)&number,sizeof(float));
}

int	client::writeToServer(double number) {
	return (int)write(serversock,(void *)&number,sizeof(double));
}

int	client::writeToServer(char character) {
	return (int)write(serversock,(void *)&character,sizeof(char));
}

int	client::writeToServer(char *string) {
	return (int)write(serversock,(void *)string,
					strlen(string)*sizeof(char));
}

int	client::writeToServer(char *string, int size) {
	return (int)write(serversock,(void *)string,size);
}

int	client::writeToServer(void *buffer, int size) {
	return (int)write(serversock,(void *)buffer,size);
}

int	client::readFromServer(short *buffer) {
	int	retval=(int)read(serversock,(void *)buffer,sizeof(short));
	*buffer=ntohs(*buffer);
	return retval;
}

int	client::readFromServer(long *buffer) {
	int	retval=(int)read(serversock,(void *)buffer,sizeof(long));
	*buffer=ntohl(*buffer);
	return retval;
}

int	client::readFromServer(float *buffer) {
	return (int)read(serversock,(void *)buffer,sizeof(float));
}

int	client::readFromServer(double *buffer) {
	return (int)read(serversock,(void *)buffer,sizeof(double));
}

int	client::readFromServer(char *buffer, int size) {
	return (int)read(serversock,(void *)buffer,size);
}

int	client::readFromServer(void *buffer, int size) {
	return (int)read(serversock,buffer,size);
}


void	client::readFromServer(strstream *buffer, char *terminator) {
 
	int	filedescriptor=serversock;
	#include "readfromfileptr.C"
}

int	client::getServerSocket() {
	return serversock;
}
