// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <iostream.h>

#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
#ifdef HAVE_SYS_WAIT_H
	#include <sys/wait.h>
#endif

#include <daemon.h>

#ifdef __GNUC__
static	signalhandler	*daemon::termhandler;
static	signalhandler	*daemon::inthandler;
static	void		(*daemon::shutdownfunc)(int);
#endif

daemon::~daemon() {
	waitOnChildren();
}

void	daemon::detach() {

	// fork and let the parent process exit
	if (fork()) {
		_exit(0);
	}
	
	// become process group and session group leader 
	// with no controlling terminal
	setsid();

	// change directory to root to avoid keeping any directories in use
	chdir("/");

	// Set umask such that files are created 666 and directories 777.
	// This way we can change them to whatever we like using chmod().
	// We want to avoid inheriting a umask which wouldn't give us write
	// permissions to files we create.
	umask(0);
}

void	daemon::handleShutDown(void *shutdownfunction) {

	shutdownfunc=(void(*)(int))shutdownfunction;

	termhandler=new signalhandler();
	termhandler->setHandler((void *)shutDown);
	termhandler->setSignal(SIGTERM);

	inthandler=new signalhandler();
	inthandler->setHandler((void *)shutDown);
	inthandler->setSignal(SIGINT);
}

void	daemon::waitOnChildren() {
	while(waitpid(-1,NULL,WNOHANG)>0);
}


void	daemon::shutDown() {

	delete termhandler;
	delete inthandler;

	(*shutdownfunc)(0);
}

int	daemon::runAsUser(char *username) {
	struct	passwd	*ent=getpwnam(username);
	if (!ent) {
		return 0;
	}
	return runAsUserId(ent->pw_uid);
}

int	daemon::runAsUserId(uid_t uid) {
	return setuid(uid);
}

int	daemon::runAsGroup(char *groupname) {
	struct	group	*ent=getgrnam(groupname);
	if (!ent) {
		return 0;
	}
	return runAsGroupId(ent->gr_gid);
}

int	daemon::runAsGroupId(gid_t gid) {
	return setgid(gid);
}
