// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#ifndef __CYGWIN__

#include <config.h>

#include <iostream.h>

#include <sys/types.h>
#include <errno.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <stdlib.h>
#include <sys/stat.h>

#include <semaphoreset.h>

#ifndef HAVE_SEMUN
union semun {
	int	val;
	struct	semid_ds	*buf;
	ushort	*array;
};
#endif

semaphoreset::semaphoreset(key_t key, 
				int semcount, int *values, 
				mode_t mode) {

	this->semcount=semcount;

	waitop=new sembuf *[semcount];
	waitwithundoop=new sembuf *[semcount];
	signalop=new sembuf *[semcount];
	signalwithundoop=new sembuf *[semcount];

	for (int i=0; i<semcount; i++) {

		// wait without undo
		waitop[i]=new sembuf[1];
		waitop[i][0].sem_num=(short)i;
		waitop[i][0].sem_op=-1;
		waitop[i][0].sem_flg=0;

		// wait with undo
		waitwithundoop[i]=new sembuf[1];
		waitwithundoop[i][0].sem_num=(short)i;
		waitwithundoop[i][0].sem_op=-1;
		waitwithundoop[i][0].sem_flg=SEM_UNDO;

		// signal without undo
		signalop[i]=new sembuf[1];
		signalop[i][0].sem_num=(short)i;
		signalop[i][0].sem_op=1;
		signalop[i][0].sem_flg=0;

		// signal with undo
		signalwithundoop[i]=new sembuf[1];
		signalwithundoop[i][0].sem_num=(short)i;
		signalwithundoop[i][0].sem_op=1;
		signalwithundoop[i][0].sem_flg=SEM_UNDO;
	}

	removeflag=0;

	// create the semaphore
	if ((semid=semget(key,semcount,IPC_CREAT|IPC_EXCL|mode))>=0) {

		// if creation succeeded, initialize the semaphore
		if (values) {
			for (int i=0; i<semcount; i++) {
				setValue(i,values[i]);
			}
		}

		// mark for removal
		removeflag=1;
		
	} else if (!(errno==EEXIST && (semid=semget(key,semcount,mode))>=0)) {

		// failure
		cerr << "error: couldn't create or access semaphore: ";
		cerr << strerror(errno) << endl;
	}
}

semaphoreset::~semaphoreset() {

	delete[] waitop;
	delete[] waitwithundoop;
	delete[] signalop;
	delete[] signalwithundoop;

	if (removeflag) {
		remove();
	}
}

int	semaphoreset::wait(int which) {
	return semop(semid,waitop[which],1);
}

int	semaphoreset::waitWithUndo(int which) {
	return semop(semid,waitwithundoop[which],1);
}

int	semaphoreset::signal(int which) {
	return semop(semid,signalop[which],1);
}

int	semaphoreset::signalWithUndo(int which) {
	return semop(semid,signalwithundoop[which],1);
}

int	semaphoreset::getValue(int which) {
	semun	semctlun;
	return semctl(semid,which,GETVAL,semctlun);
}

void	semaphoreset::setValue(int which, int value) {
	semun	semctlun;
	semctlun.val=value;
	semctl(semid,which,SETVAL,semctlun);
}

int	semaphoreset::getWaitingForZero(int which) {
	semun	semctlun;
	return semctl(semid,which,GETZCNT,semctlun);
}

int	semaphoreset::getWaitingForIncrement(int which) {
	semun	semctlun;
	return semctl(semid,which,GETNCNT,semctlun);
}

void	semaphoreset::remove() {
	semun	semctlun;
	for (int i=0; i<semcount; i++) {
		semctl(semid,i,IPC_RMID,semctlun);
	}
}

int	semaphoreset::setUid(ushort uid) {
	semun		semctlun;
	semid_ds	setds;
	setds.sem_perm.uid=uid;
	semctlun.buf=&setds;
	return semctl(semid,0,IPC_SET,semctlun);
}

int	semaphoreset::setGid(ushort gid) {
	semun		semctlun;
	semid_ds	setds;
	setds.sem_perm.gid=gid;
	semctlun.buf=&setds;
	return semctl(semid,0,IPC_SET,semctlun);
}

int	semaphoreset::setMode(mode_t mode) {
	semun		semctlun;
	semid_ds	setds;
	setds.sem_perm.mode=mode;
	semctlun.buf=&setds;
	return semctl(semid,0,IPC_SET,semctlun);
}

ushort	semaphoreset::getUid() {
	semun		semctlun;
	semid_ds	getds;
	semctlun.buf=&getds;
	semctl(semid,0,IPC_STAT,semctlun);
	return getds.sem_perm.uid;
}

ushort	semaphoreset::getGid() {
	semun		semctlun;
	semid_ds	getds;
	semctlun.buf=&getds;
	semctl(semid,0,IPC_STAT,semctlun);
	return getds.sem_perm.gid;
}

mode_t	semaphoreset::getMode() {
	semun		semctlun;
	semid_ds	getds;
	semctlun.buf=&getds;
	semctl(semid,0,IPC_STAT,semctlun);
	return getds.sem_perm.mode;
}

#endif
