// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#ifndef __CYGWIN__

#include <config.h>

#include <iostream.h>

#include <sys/types.h>
#include <errno.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <stdlib.h>
#include <sys/stat.h>

#include <sharedmemory.h>

sharedmemory::sharedmemory(key_t key, int size, mode_t mode) {

	removeflag=0;
	shmptr=NULL;

	// create the shared memory segment
	if ((shmid=shmget(key,size,IPC_CREAT|IPC_EXCL|mode))>=0) {

		// mark for removal
		removeflag=1;

		// attach to the segment
		shmptr=shmat(shmid,0,0);

		// init the segment to zero's
		memset((void *)shmptr,0,size);
		
	} else if (errno==EEXIST && (shmid=shmget(key,0,mode))>=0) {

		// attach to the segment
		shmptr=shmat(shmid,0,0);

	} else {
		// failure
		cerr << "error: couldn't create or access shared memory: ";
		cerr << strerror(errno) << endl;
	}
}

sharedmemory::~sharedmemory() {
	if (removeflag) {
		remove();
	}
}

void	*sharedmemory::getPointer() {
	return shmptr;
}

void	sharedmemory::remove() {
	shmctl(shmid,IPC_RMID,NULL);
}

int	sharedmemory::setUid(ushort uid) {
	shmid_ds	setds;
	setds.shm_perm.uid=uid;
	return shmctl(shmid,IPC_SET,&setds);
}

int	sharedmemory::setGid(ushort gid) {
	shmid_ds	setds;
	setds.shm_perm.gid=gid;
	return shmctl(shmid,IPC_SET,&setds);
}

int	sharedmemory::setMode(mode_t mode) {
	shmid_ds	setds;
	setds.shm_perm.mode=mode;
	return shmctl(shmid,IPC_SET,&setds);
}

ushort	sharedmemory::getUid() {
	shmid_ds	getds;
	shmctl(shmid,IPC_STAT,&getds);
	return getds.shm_perm.uid;
}

ushort	sharedmemory::getGid() {
	shmid_ds	getds;
	shmctl(shmid,IPC_STAT,&getds);
	return getds.shm_perm.gid;
}

mode_t	sharedmemory::getMode() {
	shmid_ds	getds;
	shmctl(shmid,IPC_STAT,&getds);
	return getds.shm_perm.mode;
}

#endif
