// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <stddef.h>

#include <signalhandler.h>

signalhandler::signalhandler() {
	handlerstruct=new (struct sigaction);
	noMasks();
	noFlags();
}

signalhandler::~signalhandler() {
	delete handlerstruct;
}

void	signalhandler::setHandler(void *handler) {
	#ifdef SIGNAL_HANDLER_INT
		handlerstruct->sa_handler=(void(*)(int))handler;
	#else
		handlerstruct->sa_handler=(void(*)(void))handler;
	#endif
}

void	signalhandler::noFlags() {
	handlerstruct->sa_flags=0;
}

void	signalhandler::addFlag(int flag) {
	handlerstruct->sa_flags|=flag;
}

int	signalhandler::noMasks() {
	return sigemptyset(&handlerstruct->sa_mask);
}

int	signalhandler::allMasks() {
	return sigfillset(&handlerstruct->sa_mask);
}

int	signalhandler::addMask(int signum) {
	return sigaddset(&handlerstruct->sa_mask,signum);
}

int	signalhandler::removeMask(int signum) {
	return sigdelset(&handlerstruct->sa_mask,signum);
}

int	signalhandler::handlesMask(int signum) {
	return sigismember(&handlerstruct->sa_mask,signum);
}

int	signalhandler::setSignal(int signum) {
	return sigaction(signum,handlerstruct,(struct sigaction *)NULL);
}

int	signalhandler::setSignal(int signum,signalhandler *oldhandler) {
	return sigaction(signum,handlerstruct,oldhandler->handlerstruct);
}

sigset_t	signalhandler::getMask() {
	return handlerstruct->sa_mask;
}

int		signalhandler::getFlags() {
	return handlerstruct->sa_flags;
}
